"""
A collection of decorators for automatically decorating functions and methods
with sphinx documentation tags.
"""
import six


def params(**argtypes):
    """
    A decorator that automatically adds parameter and return type tags to a
    function or method docstring.

    Input should be specified as the parameter keyword/name and then a tuple of
    its type and a description string. This will also use the `returns` keyword
    to identify the returned type and documentation. The following is an
    example of how to use `docparams` to automatically document arguments::

        @params(input=(str, "string to be made lowercase.")
                returns=(str, "lowercase string"))
        def lowercase(input):
            return input.lower()

    This decorator subverts the normal way decorators work to modify what is
    decorated without incuring extra overhead at call time. This means that the
    decorated function is not wrapped as a normal decorated function is, simply
    altered.

    .. seealso:: For example output see :py:func:`params_example`
    """
    def modify(function):
        def paramline(name, atype, doc):
            """Takes name, atype nd doc converts to a param docstring line."""
            return ":param {1} {0}: {2}".format(name, atype.__name__, doc)

        rtype, rdoc = argtypes.pop("returns", (None, None))
        paramlines = [paramline(name, atype, doc) for name, (atype, doc) in
                      six.iteritems(argtypes)]

        if rdoc:
            paramlines.append(":return: " + rdoc)
        if rtype:
            paramlines.append(":rtype: " + rtype.__name__)

        doc = '\n'.join(paramlines)

        if function.__doc__:
            function.__doc__ = doc + "\n\n" +  function.__doc__
        else:
            function.__doc__ = doc
        return function
    return modify


@params(text=(str, "string to make lowercase"),
        returns=(str, "lowercased text"))
def params_example(text):
    """This is a docstring."""
    return text.lower()



import sys
from contextlib import contextmanager
from functools import wraps


def get_filelike(filepath=None):
    """
    Convert a filepath into a, hopefully, appropriate file like object.

    If ``filepath`` is ``None`` then a null file like object will be returned
    that be written to but does nothing.

    If ``filepath`` is a string then a file object will be opened using
    ``filepath`` as the path to the file.

    Otherwise this function will just return the inputted ``filepath`` in the
    hope that it is file like enough.
    """
    if not filepath:
        class Devnull(object):
            def write(self, _):
                pass
        return Devnull()
    elif isinstance(filepath, str):
        return open(filepath, 'w')
    else:
        return filepath


@contextmanager
def pipe_stdout(filepath=None):
    """
    A context manager that pipes ``stdout`` to ``filepath`` after it is passed
    through :py:func:`get_filelike`.

    This can be used like so::
        with pipe_stdout():
            print("This will go no where!")
    """
    sys.stdout = get_filelike(filepath)
    yield
    sys.stdout = sys.__stdout__


@contextmanager
def pipe_stderr(filepath=None):
    """
    A context manager that pipes ``stderr`` to ``filepath`` after it is passed
    through :py:func:`get_filelike`.

    This can be used similarly to :py:func:`pipe_stdout`.
    """
    sys.stderr = get_filelike(filepath)
    yield
    sys.stderr = sys.__stderr__


@contextmanager
def pipe_std(filepath=None):
    """
    A context manager that pipes ``stdout`` and ``stderr`` to ``filepath``
    after it is passed through :py:func:`get_filelike`.

    This can be used similarly to :py:func:`pipe_stdout`.
    """
    output = get_filelike(filepath)
    sys.stdout = output
    sys.stderr = output
    yield
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__


def redirect_stdout(filepath=None):
    """
    A function wrapper that pipes ``stdout`` to ``filepath`` after it is passed
    through :py:func:`get_filelike`.

    This can be used on method or functions::
        @redirect_stdout():
        def foo():
            print("This will go no where!")
    """
    def _wrap(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            with pipe_stdout(filepath):
                function(*args, **kwargs)
        return wrapper
    return _wrap


def redirect_stderr(filepath=None):
    """
    A function wrapper that pipes ``stderr`` to ``filepath`` after it is passed
    through :py:func:`get_filelike`.

    This can be used the same way as :py:func:`redirect_stdout`.
    """
    def _wrap(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            with pipe_stderr(filepath):
                function(*args, **kwargs)
        return wrapper
    return _wrap


def redirect_std(filepath=None):
    """
    A function wrapper that pipes ``stdout`` and ``stderr`` to ``filepath``
    after it is passed through :py:func:`get_filelike`.

    This can be used the same way as :py:func:`redirect_stdout`.
    """
    def _wrap(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            with pipe_std(filepath):
                function(*args, **kwargs)
        return wrapper
    return _wrap



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_docdecs
------------

Tests for `nekrobox.docdecs` module.
"""
from nekrobox import docdecs


class TestNekrobox(object):
    def test_params(self):
        @docdecs.params(word=(str, 'echo this word'))
        def echo(word):
            return word
        assert echo.__doc__ == ":param str word: echo this word"

    def test_params_with_return_type_doc(self):
        @docdecs.params(text=(str, 'text to make lowercase'),
                        returns=(str, 'lowercase version of text'))
        def lowercase(text):
            return text.lower()
        assert lowercase.__doc__ == """:param str text: text to make lowercase
:return: lowercase version of text
:rtype: str"""



#!/usr/bin/env python

import os
import sys
from setuptools.command.test import test as TestCommand
import nekrobox

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the
        #eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


readme = open('README.rst').read()
doclink = """
Documentation
-------------

The full documentation is at http://nekrobox.rtfd.org."""
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='nekrobox',
    version=nekrobox.__version__,
    description='A utilities library by Nekroze.',
    long_description=readme + '\n\n' + doclink + '\n\n' + history,
    author='Taylor "Nekroze" Lawson',
    author_email='nekroze@eturnilnetwork.com',
    url='https://github.com/Nekroze/nekrobox',
    packages=[
        'nekrobox',
    ],
    package_dir={'nekrobox': 'nekrobox'},
    include_package_data=True,
    install_requires=[
    ],
    license='MIT',
    zip_safe=False,
    keywords='nekrobox',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    tests_require=['pytest>=2.3.5'],
    cmdclass = {'test': PyTest},
)
